@namespace html url(http://www.w3.org/1999/xhtml);

html|object:not([width]), html|object[width=""],
html|embed:not([width]), html|embed[width=""],
html|applet:not([width]), html|applet[width=""] {
  width: 240px;
}

html|object:not([height]), html|object[height=""],
html|embed:not([height]), html|embed[height=""],
html|applet:not([height]), html|applet[height=""] {
  height: 200px;
}

:-moz-type-unsupported .mainBox,
:-moz-handler-disabled .mainBox,
:-moz-handler-blocked .mainBox {
  -moz-user-focus: normal;
}
:-moz-type-unsupported .mainBox:focus,
:-moz-handler-disabled .mainBox:focus,
:-moz-handler-blocked .mainBox:focus {
  outline: 1px dotted;
}

.mainBox {
  width: inherit;
  height: inherit;
  overflow: hidden;
  direction: ltr;
  unicode-bidi: embed;
}

.mainBox[chromedir="rtl"] {
  direction: rtl;
}

.msg {
  display: none;
}

:-moz-type-unsupported .msgUnsupported,
:-moz-handler-disabled .msgDisabled,
:-moz-handler-disabled .msgManagePlugins,
:-moz-handler-blocked .msgBlocked,
:-moz-handler-crashed .msgCrashed {
  display: block;
}

.installStatus[status="ready"] .msgInstallPlugin {
  display: block;
}

.submitStatus[status="noReport"]   .msgNoCrashReport,
.submitStatus[status="please"]     .msgPleaseSubmit,
.submitStatus[status="noSubmit"]   .msgNotSubmitted,
.submitStatus[status="submitting"] .msgSubmitting,
.submitStatus[status="success"]    .msgSubmitted,
.submitStatus[status="failed"]     .msgSubmitFailed,
.submitStatus[status]:not([status="please"]) .msgReload {
  display: block;
}
.submitStatus[status="please"] .msgReload {
  /* Take up space when invisible, so stuff doesn't shift upon reveal. */
  display: block;
  visibility: hidden;
}

.helpIcon {
  cursor: pointer;
}
